<?php

namespace App\Exports;

use App\Models\DivisionModel;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Events\AfterSheet;
use Illuminate\Support\Facades\DB;
use App\Models\VehicleTrakingReportModel;
use App\Models\VehicleModel;

class DailyWeightExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['division_id']     = session::has('division_id') ? session::get('division_id') : 0;
        $aData['ward_id']         = session::has('ward_id') ? session::get('ward_id') : 0;
        $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
        $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
        $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));
        $aData['weight_status']   = session::has('weight_status') ? session::get('weight_status') : '';

        $aData['weightData']      = VehicleTrakingReportModel::get_daily_weight_mngmt($aData);
        $total_loaded = 0;
        $total_unloaded = 0;
        $total_net = 0;

        if( $aData['weightData']){
            foreach($aData['weightData'] as $key=>$aRec){ 
                $total_loaded += $aRec->loaded_weight;
                $total_unloaded += $aRec->unloaded_weight;
                $total_net += $aRec->net_weight;
                               $aData['excel'][] = array(
                        'date' => date('h:i a',$aRec->timestamp),
                        'vehicle_number' => $aRec->vehicle_number ,
                        'loaded_weight' => $aRec->loaded_weight ,
                        'unloaded_weight' => $aRec->unloaded_weight,
                        'net_weight' => $aRec->net_weight,
                        'weight_status' => 'Accept'
                    ); 
            }
            $aData['excel'][] = array(
                'Total' => "Total",
                '',
                'total_loaded' => $total_loaded,
                'total_unloaded' => $total_unloaded,
                'total_net' => $total_net
            ); 
        }else{
            $aData['excel'][] = array(
            );     
        }

        return $aData['excel'];
    }

    public function headings(): array
    {   
        $aData['division_id']     = session::has('division_id') ? session::get('division_id') : 0;
        $aData['ward_id']         = session::has('ward_id') ? session::get('ward_id') : 0;
        $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
        $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
        $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));
        $aData['weight_status']   = session::has('weight_status') ? session::get('weight_status') : '';

        $division_name = "-";
        if($aData['division_id'] != 0){
            $div = DivisionModel::where('id', $aData['division_id'])->get('division_name');
                foreach($div as $val){
                    $division_name = $val->division_name;
                }
        }

        $vehicle_number = "-";
        if($aData['vehicle_id'] != 0){
            $vehicle= VehicleModel::where('id', $aData['vehicle_id'])->get('vehicle_number');
            foreach($vehicle as $val){
                    $vehicle_number = $val->vehicle_number;
                }
        }

        $vehicle_types = "-";
        if($aData['vehicle_type_id'] != 0){
                $vehicle = DB::table('vehicle_types')->where('id',  $aData['vehicle_type_id'])->get('type'); 
                foreach($vehicle as $val){
                        $vehicle_types = $val->type;
                }
        }
               
        return [
                ['Report Name: Daily All Vehicle Weight Report.'],
                ['Division Name : '. $division_name ],
                ['Vehicle Number : '.$vehicle_number ],
                ['Vehicle Type : '.$vehicle_types ],
                ['Date : '.  $aData['date']  ],
                [],
                [ 
                    'Time ',
                    'Vehicle Number ',
                    'Loaded Weight ',
                    'Unloaded Weight ',                   
                    'Net Weight ',
                    'Status '                  
                ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');
                $event->sheet->mergeCells('A1:D1');
                $event->sheet->mergeCells('A2:D2');
                $event->sheet->mergeCells('A3:D3');
                $event->sheet->mergeCells('A4:D4');
                $event->sheet->mergeCells('A5:D5');
                $event->sheet->mergeCells('A6:Z6');
               // $event->sheet->mergeCells('A7:Z7');

                $aData = array();
                $aData['division_id']     = session::has('division_id') ? session::get('division_id') : 0;
                $aData['ward_id']         = session::has('ward_id') ? session::get('ward_id') : 0;
                $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
                $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
                $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));
                $aData['weight_status']   = session::has('weight_status') ? session::get('weight_status') : '';

                $aData['weightData']      = VehicleTrakingReportModel::get_daily_weight_mngmt($aData);
                   $i = 8;
               
                if( $aData['weightData']){
                    $data = count($aData['weightData']);
                    $i += $data;
                    $event->sheet->getStyle('A'.$i.':Z'.$i)->applyFromArray([
                        'font'=>[
                                'bold'=>true,
                        ]
                    ]);
                    $event->sheet->mergeCells('A'.$i.':B'.$i);
                    $event->sheet->getDelegate()->getStyle('A'.$i.':B'.$i)->getAlignment()->setHorizontal('right');
                }

                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('5')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('7')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('8')->setRowHeight(25);
  
                $event->sheet->getStyle('A1:Z7')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
            },
        ];
    }
  
}

