<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;

class ReportModel extends Model
{
    use HasFactory;
    protected $table = 'reports';

    public static function get_all_employee($aData) {
        $query = DB::table('biometric_users as bu')
                        ->leftJoin('biometric_image as bi','bi.user_id', '=','bu.id')
                        ->leftJoin('admin_pakages as ad','bu.Pakage_id', '=','ad.id')
                        ->leftJoin('master_designation as md','bu.designation', '=','md.id')
                        ->select('bu.*','ad.Pakage_name','md.name as designation_name',
                                'bi.image_url',
                            DB::raw('CONCAT(firstname," ", lastname) AS name')
                        );
                            
                        if($aData['Pakage_id'] > 0){
                            $query->where('ad.id',$aData['Pakage_id']);
                        } 
                        
                        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
                            $session_div = Session::get('Pakages'); 
                            if(!empty($session_div)){
                                $query->whereIn('bu.Pakage_id',$session_div);    
                            }else {
                                $query->whereRaw('1=2');
                            }
                        }     
        return $query;
    }

    public static function get_employee_attendance($aData){

        $start_date = $aData['attendance_date'] ? $aData['attendance_date'].' 00:00:00' : date('Y-m-d 00:00:00');
        $end_date = $aData['attendance_date'] ? $aData['attendance_date'].' 23:59:59' : date('Y-m-d 23:59:59');
        
        $query = DB::table('users_attendance as ua')
                ->leftJoin('admin_pakages', 'admin_pakages.id', 'ua.Pakage_id')
                ->leftJoin('user_attendance_history as t1', 't1.id','=','ua.inward_success_id')
                ->leftJoin('user_attendance_history as t2', 't2.id','=','ua.outword_success_id')
                ->leftJoin('biometric_users', 'biometric_users.id', 'ua.user_id')
                ->select('ua.id',
                        'ua.attendance_date',
                        'ua.inward_datetime',
                        'ua.outword_datetime',
                        'ua.success',
                        'admin_pakages.Pakage_name', 
                        'biometric_users.emp_id', 
                        DB::raw('(SELECT supervisor_name FROM supervisor_masters WHERE id= ua.supervisor_id) AS supervisor_name' ),
                        DB::raw('CONCAT(biometric_users.firstname," ", biometric_users.lastname) AS name'),
                        't1.location as in_location',
                        't2.location as out_location',
                        't1.image_url as in_image_url',
                        't2.image_url as out_image_url',  
                    );
                $query->where('ua.report_invalid',0);
                if($aData['Pakage_id'] > 0){
                    $query->where('ua.Pakage_id',$aData['Pakage_id']);
                } 

                if($start_date > 0 && $end_date > 0){
                    $query->whereBetween('ua.attendance_date',[strtotime($start_date), strtotime($end_date)]);
                } 

                if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
                    $session_div = Session::get('Pakages'); 
                    if(!empty($session_div)){
                        $query->whereIn('ua.Pakage_id',$session_div);    
                    }else {
                        $query->whereRaw('1=2');
                    }
                }     
        
        return $query;
    }

    public static function getTotalHours($aPost){
        $iPakageID = (isset($aPost['Pakage_id'])) ? $aPost['Pakage_id'] : 0;
		$sStartDate = (isset($aPost['start_date'])) ? $aPost['start_date'] : 0;
		$sEndDate = (isset($aPost['end_date'])) ? $aPost['end_date'] : 0;
		
		$iStartDate = strtotime($sStartDate);
		$iStartDate = $iStartDate - 10;
		$iEndDate = strtotime($sEndDate.' +1 day');
		
		$sSQL ='select SUM((outword_datetime-inward_datetime)) as hours	from users_attendance ua where ua.report_invalid=0 and  ua.success=1 and ua.attendance_date>'.$iStartDate.' and ua.attendance_date<'.$iEndDate.' and inward_datetime <= outword_datetime';
			
		if($iPakageID>0){
            $sSQL .= ' and ua.Pakage_id='.$iPakageID;
        }

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND ua.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }

        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;
    }

    public static function getHoursData($aPost){
        $iPakageID = (isset($aPost['Pakage_id'])) ? $aPost['Pakage_id'] : 0;
		$sStartDate = (isset($aPost['start_date'])) ? $aPost['start_date'] : 0;
		$sEndDate = (isset($aPost['end_date'])) ? $aPost['end_date'] : 0;
		
		$iStartDate = strtotime($sStartDate);
		$iStartDate = $iStartDate - 10;
		$iEndDate = strtotime($sEndDate.' +1 day');

        $sSQL ='select ua.user_id,SUM((outword_datetime-inward_datetime)) as hours,
				(select image_url from biometric_image where user_id=ua.user_id LIMIT 1) as image_url,
				(select CONCAT(`firstname`,\' \',`middlename`,\' \',`lastname`) as name from biometric_users where id=ua.user_id) as employee,
				(select emp_id from biometric_users where id=ua.user_id) as emp_id,
                ua.Pakage_id
				from users_attendance ua  where ua.report_invalid=0 and  ua.success=1 and ua.attendance_date>'.$iStartDate.' and ua.attendance_date<'.$iEndDate.' and inward_datetime < outword_datetime';

		if($iPakageID>0){
            $sSQL .= ' and ua.Pakage_id='.$iPakageID;
        }

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND ua.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }

        $sSQL .= ' GROUP by ua.user_id';
		$sSQL .= ' ORDER by emp_id';
        
        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;
    }

    public static function getPresentData($aPost){

        $iPakageID = (isset($aPost['Pakage_id'])) ? $aPost['Pakage_id'] : 0;		
		$sStartDate = (isset($aPost['start_date'])) ? $aPost['start_date'] : 0;
		$sEndDate = (isset($aPost['end_date'])) ? $aPost['end_date'] : 0;
		
		$iStartDate = strtotime($sStartDate);
		$iStartDate = $iStartDate - 10;
		$iEndDate = strtotime($sEndDate.' +1 day');
		
		$sSQL ='select COUNT(ua.id) as present_day,
				(select image_url from biometric_image where user_id=ua.user_id LIMIT 1) as image_url,
				(select CONCAT(`firstname`,\' \',`middlename`,\' \',`lastname`) as name from biometric_users where id=ua.user_id) as employee,
				(select emp_id from biometric_users where id=ua.user_id) as emp_id,
                ua.Pakage_id
				from users_attendance ua where ua.report_invalid=0 and ua.success=1 and ua.attendance_date>'.$iStartDate.' and ua.attendance_date<'.$iEndDate.'';
			
		if($iPakageID>0){
				$sSQL .= ' and ua.Pakage_id='.$iPakageID;
        }

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND ua.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }

		$sSQL .= ' GROUP by ua.user_id ';
		$sSQL .= ' ORDER by emp_id';
        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;
    }

    public static function getDaysData($aPost){
        
        $iPakageID = (isset($aPost['Pakage_id'])) ? $aPost['Pakage_id'] : 0;
		$sStartDate = (isset($aPost['start_date'])) ? $aPost['start_date'] : 0;
		$sEndDate = (isset($aPost['end_date'])) ? $aPost['end_date'] : 0;
		$iStartDate = strtotime($sStartDate);
		$iStartDate = $iStartDate - 10;
		$iEndDate = strtotime($sEndDate.' +1 day');
		$sSQL = 'select ua.attendance_date,SUM(ua.outword_datetime-ua.inward_datetime) as total,count(user_id) as employee_count from users_attendance ua where ua.report_invalid=0 and ua.success=1 and ua.attendance_date>'.$iStartDate.' and ua.attendance_date<'.$iEndDate;	
		
        if($iPakageID>0){
            $sSQL .= ' and ua.Pakage_id='.$iPakageID;
        }

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND ua.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }

        
        $sSQL .= ' GROUP by ua.attendance_date';
        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;

    }

    public static function getSingleHours($aPost){
        $sStartDate = (isset($aPost['start_date'])) ? $aPost['start_date'] : 0;
		$sEndDate = (isset($aPost['end_date'])) ? $aPost['end_date'] : 0;
		
		$iStartDate = strtotime($sStartDate);
		$iStartDate = $iStartDate - 10;
		$iEndDate = strtotime($sEndDate.' +1 day');
		
		$sSQL ='select SUM((ua.outword_datetime-ua.inward_datetime)) as hours from users_attendance ua left join biometric_users bu on bu.id=ua.user_id where ua.report_invalid=0 and ua.success=1 and ua.attendance_date>'.$iStartDate.' and ua.attendance_date<'.$iEndDate.' and ua.inward_datetime <= ua.outword_datetime and bu.emp_id= "'.$aPost['emp_id'].'"';
        
        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND ua.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }
        
        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;
    }

    public static function getEmployee($aPost){
        $aReturn = array();
		
		$sSQL ='select emp_id,CONCAT(`firstname`,\' \',`middlename`,\' \',`lastname`) as name
		,(select Pakage_name from admin_pakages where id=bu.Pakage_id) as Pakage_name from biometric_users bu where emp_id = "'.$aPost['emp_id'].'"';
        
        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND bu.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }
        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;
    }

    public static function getSingleData($aPost){
        $sStartDate = (isset($aPost['start_date'])) ? $aPost['start_date'] : 0;
		$sEndDate = (isset($aPost['end_date'])) ? $aPost['end_date'] : 0;
		
		$iStartDate = strtotime($sStartDate);
		$iStartDate = $iStartDate - 10;
		$iEndDate = strtotime($sEndDate.' +1 day');
		
		$sSQL ='select 
				ua.attendance_date,
				ua.inward_datetime,
				(select location from user_attendance_history where id=ua.inward_success_id) as in_location,
				(select image_url from user_attendance_history where id=ua.inward_success_id) as in_image_url,
				ua.outword_datetime, 
				(select location from user_attendance_history where id=ua.outword_success_id) as out_location,
				(select image_url from user_attendance_history where id=ua.outword_success_id) as out_image_url,
				ua.success,
				(ua.outword_datetime-ua.inward_datetime) as hours
				from users_attendance ua left join biometric_users bu on bu.id=ua.user_id 
                where ua.report_invalid=0 and 
                ua.attendance_date>'.$iStartDate.' and 
                ua.attendance_date<'.$iEndDate.'  and 
                bu.emp_id="'.$aPost['emp_id'].'"';
                
                if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
                    $session_div = Session::get('Pakages'); 
                    if(!empty($session_div)){
                        $str_div = implode(',',$session_div);
                        $sSQL .= ' AND ua.Pakage_id IN ('.$str_div.')';
                    }else {
                        $sSQL .= ' AND 1=2';
                    }
                }
        
        $sSQL .= ' order by ua.Pakage_id,ua.user_id,ua.attendance_date';
        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;

    }
    public static function reportInvalid($aIds){
        
        if(!empty($aIds)){
           
            $sIds = implode(',',$aIds);
            
            $sSQL = 'update users_attendance set report_invalid=1 where id IN ('.$sIds.')';
			DB::select( DB::raw($sSQL));
        }
    }

    public static function reportValidate($aIds){
        
        if(!empty($aIds)){
           
            $sIds = implode(',',$aIds);
            
            $sSQL = 'update users_attendance set report_invalid=0 where id IN ('.$sIds.')';
			DB::select( DB::raw($sSQL));
        }
    }

    

    public static function getInvalidData($aPost){
        $iPakageID = (isset($aPost['Pakage_id'])) ? $aPost['Pakage_id'] : 0;
		$sStartDate = (isset($aPost['start_date'])) ? $aPost['start_date'] : 0;
		$sEndDate = (isset($aPost['end_date'])) ? $aPost['end_date'] : 0;
		
		$iStartDate = strtotime($sStartDate);
		$iStartDate = $iStartDate - 10;
		$iEndDate = strtotime($sEndDate.' +1 day');
		
		$sSQL ='select 
				ua.id,
				ua.attendance_date,
				ua.inward_datetime,
				(select location from user_attendance_history where id=ua.inward_success_id) as in_location,
				(select image_url from user_attendance_history where id=ua.inward_success_id) as in_image_url,
				ua.outword_datetime, 
				(select location from user_attendance_history where id=ua.outword_success_id) as out_location,
				(select image_url from user_attendance_history where id=ua.outword_success_id) as out_image_url,
				(select CONCAT(`firstname`,\' \',`middlename`,\' \',`lastname`) as name from biometric_users where id=ua.user_id) as employee,
				(select emp_id from biometric_users where id=ua.user_id) as emp_id,
				(select Pakage_name from admin_pakages where id=ua.Pakage_id) as Pakage_name,
				ua.success,
				(ua.outword_datetime-ua.inward_datetime) as hours
				from users_attendance ua 
                where ua.report_invalid=1 and 
                ua.attendance_date>'.$iStartDate.' and 
                ua.attendance_date<'.$iEndDate.'';
				
		if($iPakageID>0){
            $sSQL .= ' and ua.Pakage_id='.$iPakageID;	
        }

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND ua.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }

        $sSQL .= ' order by emp_id asc,ua.attendance_date desc,ua.inward_datetime desc,ua.user_id';
        $rRes = DB::select( DB::raw($sSQL));
        return $rRes;
    }
  
    public static function datewisePresentAbsentCount($aPost){
		$start_date = strtotime($aPost['start_date']);
		$end_date = strtotime($aPost['end_date']);
        /*$totalUsersSql = 'select count(bu.id) as totalUsers from biometric_users bu where 1=1';
        
        if(!empty($aPost['Pakage_id'])){
            $totalUsersSql .= ' and bu.Pakage_id = '.$aPost['Pakage_id'];
        }

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $totalUsersSql .= ' AND bu.Pakage_id IN ('.$str_div.')';
            }else {
                $totalUsersSql .= ' AND 1=2';
            }
        }
        
       // echo $totalUsersSql; die;
        $result = DB::select( DB::raw($totalUsersSql));
        $totalUserCount = "";
       
        if(!empty($result)){
            $totalUserCount = $result[0]->totalUsers;
        }
        */
       
        $start_day = date('Ymd',$start_date);
        $end_day = date('Ymd',$end_date);
        $sql = 'SELECT day,vehicle_type_id,no_of_vehicles FROM daily_total_vehicles WHERE day>='.$start_day.' AND day<='.$end_day;
        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sql .= ' AND Pakage_id in ('.$str_div.') ';
            }else {
                $query->whereRaw('1=2');
            }
        }
        if(!empty($aPost['Pakage_id'])){
            $sql .= ' AND Pakage_id = '.$aPost['Pakage_id'];
        }
        
        $result = DB::select( DB::raw($sql));
        $TotalCounts = array();
        foreach($result as $Daily){
            
            $total = 0;
            if($Daily->vehicle_type_id == 6){
                $total = $Daily->no_of_vehicles * 2;
            }else{
                $total = $Daily->no_of_vehicles * 3;
            }
            if(!empty($TotalCounts[$Daily->day]))
                $TotalCounts[$Daily->day] += $total;
            else
                $TotalCounts[$Daily->day] = $total;
        }
        //dd($TotalCounts);
        $query = DB::table('users_attendance as ua')
                    ->select(DB::raw("COUNT('ua.id') as presentCount"), 'ua.attendance_date')
                    ->where('ua.success',1);
                                
                    $query->where('report_invalid',0);
                    if(!empty($aPost['Pakage_id'])){
                            $query->where('ua.Pakage_id',$aPost['Pakage_id']);
                    }
                    
                    if(!empty($start_date) && !empty($end_date)){
                        $query->whereBetween('ua.attendance_date',[$start_date, $end_date]);
                    }
        
                    if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
                        $session_div = Session::get('Pakages'); 
                        if(!empty($session_div)){
                            $query->whereIn('ua.Pakage_id',$session_div);    
                        }else {
                            $query->whereRaw('1=2');
                        }
                    }
        
                    $aData = array();
        $aData['data'] = $query->groupBy('ua.attendance_date')->get();
        //dd($aData['data']);
        foreach( $aData['data'] as $key=>$val){
            //dd($key);
            $curr_date = date('Ymd',$val->attendance_date);
            $totalUserCount = (!empty($TotalCounts[$curr_date])) ? $TotalCounts[$curr_date] : 0;
            if($totalUserCount > $val->presentCount ) 
                $aData['data'][$key]->absentCount =  $totalUserCount - $val->presentCount;
            else
                $aData['data'][$key]->absentCount = 0;
        }
        return $aData['data'];
	}

    public static function getWorkingDays($startDate, $endDate,$holidays){
        $begin = strtotime($startDate);
        $end   = strtotime($endDate);
        if ($begin > $end) {
            echo "startdate is in the future! <br />";
            return 0;
        } else {
            $no_days  = 0;
            $weekends = 0;
            while ($begin <= $end) {
                $no_days++; // no of days in the given interval
                $what_day = date("N", $begin);
            
                $begin += 86400; // +1 day
            };
            $working_days = $no_days - $weekends;
            return $working_days;

        }
    }

    public static function secondsToTime($seconds)
	{
        $data=  gmdate("H:i:s", $seconds);

		// extract hours
		$hours = floor($seconds / (60 * 60));

		// extract minutes
		$divisor_for_minutes = $seconds % (60 * 60);
		$minutes = floor($divisor_for_minutes / 60);
	 
		// extract the remaining seconds
		$divisor_for_seconds = $divisor_for_minutes % 60;
		$seconds = ceil($divisor_for_seconds);
	 
		// return the final array
		$obj = array(
			"h" => (int) $hours,
			"m" => (int) $minutes,
			"s" => (int) $seconds,
		);
        // die($obj);
		return $obj;
        
	}
    

}

